package w2phtml.pageSplitters;

import static w2phtml.office.XMLString.FO_TEXT_INDENT;
import static w2phtml.office.XMLString.TEXT_NOTE_BODY;
import static w2phtml.office.XMLString.TEXT_SOFT_PAGE_BREAK;
import static w2phtml.office.XMLString.TEXT_STYLE_NAME;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import w2phtml.office.OfficeReader;
import w2phtml.office.StyleWithProperties;
import w2phtml.util.Misc;

public class BasicSplitter {
	protected OfficeReader officeReader;
	protected SplitFactory factory;
	
	public BasicSplitter(OfficeReader officeReader, SplitFactory factory) {
		this.officeReader = officeReader;
		this.factory = factory;
	}
	protected static boolean appendChild(Node parent, Node child) {
		boolean dataMoved;
		parent.appendChild(child);
		dataMoved = true;
		return dataMoved;
	}
	
	protected static void addHyphen(Node para, Node paraBefore) {
		//Debug.printNode(paraBefore);
		//Debug.printNode(para);
		Character softHyphen = 0x00ad;
		Character lastChar = getLastChar(paraBefore);
		Character firstChar = getFirstChar(para);
		
		if (lastChar == null || firstChar == null) {
			return;
		}
		Node lastNode = paraBefore;
		while (lastNode.hasChildNodes()) {
			lastNode = lastNode.getLastChild();
		}
		String lineEndsWith = lastNode.getTextContent();
		if (Character.isLetter(lastChar) && Character.isLetter(firstChar)) {
			
			lastNode.setTextContent(lastNode.getTextContent() + "\u2010");
		} else if (lastChar.equals(softHyphen)) {
			lastNode.setTextContent(lineEndsWith.substring(0, lineEndsWith.length()-1) + "\u2010");
		}
	}
	protected static Character getLastChar(Node para) {
		if (para == null) {		return null;		}
		Node lastNode = para;
		while (lastNode.hasChildNodes()) {
			lastNode = lastNode.getLastChild();
		}
		String content = lastNode.getTextContent();
		if (content != null && !content.isEmpty()) {
			return content.charAt(content.length()-1);
		}
		return null;
	}
	
	protected static Character getFirstChar(Node para) {
		if (para == null) {		return null;		}
		Node firstNode = para;
		while (firstNode.hasChildNodes()) {
			firstNode = firstNode.getFirstChild();
		}
		String content = firstNode.getTextContent();
		if (content != null && !content.isEmpty()) {
			return content.charAt(0);
		}
		return null;
	}
	
	protected void removeIndent(Node paraAfter) {
		String baseStyleName = Misc.getAttribute(paraAfter, TEXT_STYLE_NAME);
		String newStyleName = officeReader.cloneParStyle(baseStyleName);
		
		Node styleAttr = paraAfter.getAttributes().getNamedItem(TEXT_STYLE_NAME);
		styleAttr.setTextContent(newStyleName);
		
		StyleWithProperties newStyle = officeReader.getParStyle(Misc.getAttribute(paraAfter, TEXT_STYLE_NAME));
		newStyle.setParProperty(FO_TEXT_INDENT, "0");
	}
	
	protected static boolean containsSPB(Node node) {
		Node parent = node.getParentNode();
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			if (node.getNodeName().equals(TEXT_SOFT_PAGE_BREAK) && (parent.getNodeName() != TEXT_NOTE_BODY)) {
				return true;
			}
			if (node.hasChildNodes()) {
				int currentNo = 0;
				NodeList childNodes = node.getChildNodes();
				while (currentNo < childNodes.getLength()) {
					Node childNode = childNodes.item(currentNo);
					if (containsSPB(childNode)) {
						return true;
					}
					currentNo++;
				}
			}
		}
		return false;
	}
	
	protected static void removeSPB(Node node) {
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			if (node.getNodeName().equals(TEXT_SOFT_PAGE_BREAK)) {
				Node parent = node.getParentNode();
				parent.removeChild(node);
				return;
			}
			if (node.hasChildNodes()) {
				int currentNo = 0;
				NodeList childNodes = node.getChildNodes();
				while (currentNo < childNodes.getLength()) {
					Node childNode = childNodes.item(currentNo);
					removeSPB(childNode);					
					currentNo++;
				}
			}
		}
	}
}
