package w2phtml.pageSplitters;

import static w2phtml.office.XMLString.TEXT_H;
import static w2phtml.office.XMLString.TEXT_LIST;
import static w2phtml.office.XMLString.TEXT_P;
import static w2phtml.office.XMLString.TEXT_SOFT_PAGE_BREAK;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import w2phtml.office.OfficeReader;

public class ListItemSplitter extends BasicSplitter implements ISplitter {

	public ListItemSplitter(OfficeReader officeReader, SplitFactory factory) {
		super(officeReader, factory);
	}

	@Override
	public boolean Split(Node listItem) {
		Node listItemFirstPart = listItem.cloneNode(false);
		Node parent = listItem.getParentNode();
		int i = 0;
		boolean dataMoved = false;
		NodeList listItemNodes = listItem.getChildNodes();
		while(listItemNodes.getLength() > i){
			Node listItemChild = listItemNodes.item(i);
			if(listItemChild.getNodeType() == Node.ELEMENT_NODE){
				//Node name
				String nodeName = listItemChild.getNodeName();
				if (containsSPB(listItemChild)){
					//Break if SPB
					if (nodeName.equals(TEXT_SOFT_PAGE_BREAK)) {
						//Remove SPB.Return result
						listItem.removeChild(listItemChild);
					} else if (nodeName.equals(TEXT_LIST)) {
						if (factory.split(listItemChild)){
							dataMoved=true;
						}
						
					} else if (nodeName.equals(TEXT_H) || nodeName.equals(TEXT_P)) {
						if (factory.split(listItemChild)){
							listItemFirstPart.appendChild(listItemChild.getPreviousSibling());
							dataMoved=true;
						} 
					} else {
						System.out.println("Error. SPB in List item child node " + nodeName);
						System.exit(1);
					}
					break;
				//Move to first part
				} else {
					
					listItemFirstPart.appendChild(listItemChild);
					dataMoved = true;
				}
			} else {
				listItemFirstPart.appendChild(listItemChild);
				dataMoved = true;
			}
			//check internal nodes
		}
		if(dataMoved) {
			parent.insertBefore(listItemFirstPart, listItem);
		}
		return dataMoved;
	}

}
