package w2phtml.pageSplitters;

import static w2phtml.office.XMLString.TABLE_TABLE;
import static w2phtml.office.XMLString.TEXT_H;
import static w2phtml.office.XMLString.TEXT_LIST;
import static w2phtml.office.XMLString.TEXT_P;
import static w2phtml.office.XMLString.TEXT_SECTION;
import static w2phtml.office.XMLString.TEXT_SOFT_PAGE_BREAK;
import static w2phtml.office.XMLString.TEXT_TABLE_OF_CONTENT;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import w2phtml.office.OfficeReader;

public class SectionSplitter extends BasicSplitter implements ISplitter {

	public SectionSplitter(OfficeReader officeReader, SplitFactory factory) {
		super(officeReader, factory);
		// TODO Auto-generated constructor stub
	}

	@Override
	public boolean Split(Node section) {
		Node parent = section.getParentNode();
		setLastAttribute(section);
		Node sectionFirstPart = section.cloneNode(false);
		removeLastAttribute(sectionFirstPart);
		boolean dataMoved = false;
		// Node counter
		int i = 0;
		NodeList childs = section.getChildNodes();
		while (childs.getLength() > i) {
			
			Node child = childs.item(0);
			if ((child.getNodeType() == Node.ELEMENT_NODE)) {
				String nodeName = child.getNodeName();
				if (containsSPB(child)){
					if (nodeName.equals(TEXT_SOFT_PAGE_BREAK)){
						// remove inner soft page break node
						section.removeChild(child);
					} else if (nodeName.equals(TEXT_H) || nodeName.equals(TEXT_P)) {
						if (factory.split(child)){
							sectionFirstPart.appendChild(child.getPreviousSibling());
							dataMoved=true;
						}
					} else if (nodeName.equals(TEXT_TABLE_OF_CONTENT)) {
						
						//HACK
						removeSPB(section);
						i++;
						continue;
					} else if (nodeName.equals(TABLE_TABLE)) {
						if (factory.split(child)){
							sectionFirstPart.appendChild(child.getPreviousSibling());
							dataMoved=true;
						}
					} else if (nodeName.equals(TEXT_SECTION)) {
						if (factory.split(child)){
							sectionFirstPart.appendChild(child.getPreviousSibling());
							dataMoved=true;
						}
					} else if (nodeName.equals(TEXT_LIST)) {
						if (factory.split(child)){
							dataMoved=true;
							}
						} else {
							System.out.println("Error. SPB in Section child node " + nodeName);
							System.exit(1);
						}
					//split node with spb and exit
					break;
				} else {
						sectionFirstPart.appendChild(child);
						dataMoved = true;
				}
			} else {
				//Append text nodes
				sectionFirstPart.appendChild(child);
				dataMoved = true;	
			}
			
		}
		if (dataMoved) {
			parent.insertBefore(sectionFirstPart, section);
		} 
		
		return dataMoved;
	}

	
	private static void removeLastAttribute(Node section) {
		Element sectionEl = (Element) section;
		sectionEl.setAttribute("last", "false");
	}
	private static void setLastAttribute(Node section) {
		Element sectionEl = (Element) section;
		sectionEl.setAttribute("last", "true");
	}

}
