/************************************************************************
 *
 *  ConverterHelper.java
 *
 *  Copyright: 2002-2015 by Henrik Just
 *
 *  This file is part of Writer2LaTeX.
 *  
 *  Writer2LaTeX is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Writer2LaTeX is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Writer2LaTeX.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Version 1.6 (2015-06-15)
 *
 */

package w2phtml.xhtml;

import org.w3c.dom.Element;

import w2phtml.office.OfficeReader;
import w2phtml.xhtml.content.DrawParser;
import w2phtml.xhtml.content.ListParser;
import w2phtml.xhtml.content.MathParser;
import w2phtml.xhtml.content.TableParser;
import w2phtml.xhtml.content.TextParser;
import w2phtml.xhtml.style.CellStyleParser;
import w2phtml.xhtml.style.FrameStyleParser;
import w2phtml.xhtml.style.HeadingStyleParser;
import w2phtml.xhtml.style.ListStyleParser;
import w2phtml.xhtml.style.PageStyleParser;
import w2phtml.xhtml.style.ParStyleParser;
import w2phtml.xhtml.style.PresentationStyleParser;
import w2phtml.xhtml.style.RowStyleParser;
import w2phtml.xhtml.style.SectionStyleParser;
import w2phtml.xhtml.style.Styles;
import w2phtml.xhtml.style.TableStyleParser;
import w2phtml.xhtml.style.TextStyleParser;

/** A <code>ConverterHelper</code> is responsible for conversion of some specific content into XHTML. 
 */
public class Parser {
	
	// Member variables providing our content (set in constructor)
	protected OfficeReader ofr;
    protected XhtmlConfig config;
    protected Converter converter;
	
    /** Construct a new converter helper based on a 
     * 
     * @param ofr the office reader used to access the source document
     * @param config the configuration to use
     * @param converter the main converter to which the helper belongs
     */
    protected Parser(OfficeReader ofr, XhtmlConfig config, Converter converter) {
        this.ofr = ofr;
        this.config = config;
        this.converter = converter;
    }
    
    // Convenience accessor methods to other converter helpers (only needed to save some typing)

    protected Styles getStyleCv() { return converter.getStylesParser(); }

    protected TextStyleParser getTextSP() { return converter.getStylesParser().getTextSP(); }
	
    protected ParStyleParser getParSP() { return converter.getStylesParser().getParSP(); }
	
    protected HeadingStyleParser getHeadingSP() { return converter.getStylesParser().getHeadingSP(); }
	
    protected ListStyleParser getListSP() { return converter.getStylesParser().getListSP(); }
	
    protected SectionStyleParser getSectionSP() { return converter.getStylesParser().getSectionSP(); }
	
    protected TableStyleParser getTableSP() { return converter.getStylesParser().getTableSP(); }
	
    protected RowStyleParser getRowSP() { return converter.getStylesParser().getRowSP(); }
	
    protected CellStyleParser getCellSP() { return converter.getStylesParser().getCellSP(); }
	
    protected FrameStyleParser getFrameSP() { return converter.getStylesParser().getFrameSP(); }
	
    protected PresentationStyleParser getPresentationSP() { return converter.getStylesParser().getPresentationSP(); }
	
    protected PageStyleParser getPageSP() { return converter.getStylesParser().getPageSP(); }
    
    protected TextParser getTextParser() { return converter.getTextParser(); }
    
    protected ListParser getListParser() { return converter.getListParser(); }

	
    protected TableParser getTableParser() { return converter.getTableParser(); }

    protected DrawParser getDrawParser() { return converter.getDrawParser(); }

    protected MathParser getMathParser() { return converter.getMathParser(); }
	
    /** Apply style information to an XHTML node
     * 
     * @param info the style to apply
     * @param hnode the XHTML node
     */
    public void writeStyle(StyleInfo info, Element hnode) {
        if (info.sClass!=null) {
            hnode.setAttribute("class",info.sClass);
        }
        if (!info.props.isEmpty()) {
            hnode.setAttribute("style",info.props.toString());
        }
        if (info.sLang!=null) {
            hnode.setAttribute("xml:lang",info.sLang);
            if (converter.getType()==XhtmlDocument.XHTML10 || converter.getType()==XhtmlDocument.HTML5) {
                hnode.setAttribute("lang",info.sLang); // HTML4 compatibility/polyglot HTML5S
            }
        }
        if (info.sDir!=null) {
            hnode.setAttribute("dir",info.sDir);
        }
    }

}
